Copy

Trait Copy 

1.0.0 ยท Source
pub trait Copy: Clone { }
Types whose values can be duplicated simply by copying bits.

By default, variable bindings have โ€˜move semantics.โ€™ In other words:

#[derive(Debug)]
struct Foo;

let x = Foo;

let y = x;

// `x` has moved into `y`, and so cannot be used

// println!("{x:?}"); // error: use of moved value

However, if a type implements Copy, it instead has โ€˜copy semanticsโ€™:

// We can derive a `Copy` implementation. `Clone` is also required, as it's
// a supertrait of `Copy`.
#[derive(Debug, Copy, Clone)]
struct Foo;

let x = Foo;

let y = x;

// `y` is a copy of `x`

println!("{x:?}"); // A-OK!

Itโ€™s important to note that in these two examples, the only difference is whether you are allowed to access x after the assignment. Under the hood, both a copy and a move can result in bits being copied in memory, although this is sometimes optimized away.

How can I implement Copy?

There are two ways to implement Copy on your type. The simplest is to use derive:

#[derive(Copy, Clone)]
struct MyStruct;

You can also implement Copy and Clone manually:

struct MyStruct;

impl Copy for MyStruct { }

impl Clone for MyStruct {
    fn clone(&self) -> MyStruct {
        *self
    }
}

There is a small difference between the two. The derive strategy will also place a Copy bound on type parameters:

#[derive(Clone)]
struct MyStruct<T>(T);

impl<T: Copy> Copy for MyStruct<T> { }

This isnโ€™t always desired. For example, shared references (&T) can be copied regardless of whether T is Copy. Likewise, a generic struct containing markers such as PhantomData could potentially be duplicated with a bit-wise copy.

Whatโ€™s the difference between Copy and Clone?

Copies happen implicitly, for example as part of an assignment y = x. The behavior of Copy is not overloadable; it is always a simple bit-wise copy.

Cloning is an explicit action, x.clone(). The implementation of Clone can provide any type-specific behavior necessary to duplicate values safely. For example, the implementation of Clone for String needs to copy the pointed-to string buffer in the heap. A simple bitwise copy of String values would merely copy the pointer, leading to a double free down the line. For this reason, String is Clone but not Copy.

Clone is a supertrait of Copy, so everything which is Copy must also implement Clone. If a type is Copy then its Clone implementation only needs to return *self (see the example above).

When can my type be Copy?

A type can implement Copy if all of its components implement Copy. For example, this struct can be Copy:

#[derive(Copy, Clone)]
struct Point {
   x: i32,
   y: i32,
}

A struct can be Copy, and i32 is Copy, therefore Point is eligible to be Copy. By contrast, consider

struct PointList {
    points: Vec<Point>,
}

The struct PointList cannot implement Copy, because Vec<T> is not Copy. If we attempt to derive a Copy implementation, weโ€™ll get an error:

the trait `Copy` cannot be implemented for this type; field `points` does not implement `Copy`

Shared references (&T) are also Copy, so a type can be Copy, even when it holds shared references of types T that are not Copy. Consider the following struct, which can implement Copy, because it only holds a shared reference to our non-Copy type PointList from above:

#[derive(Copy, Clone)]
struct PointListWrapper<'a> {
    point_list_ref: &'a PointList,
}

When canโ€™t my type be Copy?

Some types canโ€™t be copied safely. For example, copying &mut T would create an aliased mutable reference. Copying String would duplicate responsibility for managing the Stringโ€™s buffer, leading to a double free.

Generalizing the latter case, any type implementing Drop canโ€™t be Copy, because itโ€™s managing some resource besides its own size_of::<T> bytes.

If you try to implement Copy on a struct or enum containing non-Copy data, you will get the error E0204.

When should my type be Copy?

Generally speaking, if your type can implement Copy, it should. Keep in mind, though, that implementing Copy is part of the public API of your type. If the type might become non-Copy in the future, it could be prudent to omit the Copy implementation now, to avoid a breaking API change.

Additional implementors

In addition to the implementors listed below, the following types also implement Copy:

  • Function item types (i.e., the distinct types defined for each function)
  • Function pointer types (e.g., fn() -> i32)
  • Closure types, if they capture no value from the environment or if all such captured values implement Copy themselves. Note that variables captured by shared reference always implement Copy (even if the referent doesnโ€™t), while variables captured by mutable reference never implement Copy.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl Copy for !

1.26.0 ยท Source

impl Copy for AccessError

Source

impl Copy for std::mem::Alignment

1.28.0 ยท Source

impl Copy for std::fmt::Alignment

Source

impl Copy for AllocError

Source

impl Copy for AsciiChar

Source

impl Copy for Assume

Source

impl Copy for BacktraceStyle

Source

impl Copy for CharCase

1.34.0 ยท Source

impl Copy for CharTryFromError

1.27.0 ยท Source

impl Copy for CpuidResult

Source

impl Copy for DebugAsHex

1.3.0 ยท Source

impl Copy for Duration

1.0.0 ยท Source

impl Copy for Empty

1.0.0 ยท Source

impl Copy for Error

1.0.0 ยท Source

impl Copy for ErrorKind

1.61.0 ยท Source

impl Copy for ExitCode

1.0.0 ยท Source

impl Copy for ExitStatus

Source

impl Copy for ExitStatusError

Source

impl Copy for FieldId

1.75.0 ยท Source

impl Copy for FileTimes

1.1.0 ยท Source

impl Copy for FileType

Source

impl Copy for FormattingOptions

1.0.0 ยท Source

impl Copy for FpCategory

1.69.0 ยท Source

impl Copy for FromBytesUntilNulError

1.64.0 ยท Source

impl Copy for FromBytesWithNulError

Source

impl Copy for Global

1.34.0 ยท Source

impl Copy for Infallible

1.8.0 ยท Source

impl Copy for Instant

1.55.0 ยท Source

impl Copy for IntErrorKind

1.7.0 ยท Source

impl Copy for IpAddr

1.0.0 ยท Source

impl Copy for Ipv4Addr

1.0.0 ยท Source

impl Copy for Ipv6Addr

Source

impl Copy for Ipv6MulticastScope

1.28.0 ยท Source

impl Copy for Layout

Source

impl Copy for Locality

1.0.0 ยท Source

impl Copy for std::cmp::Ordering

1.0.0 ยท Source

impl Copy for std::sync::atomic::Ordering

1.33.0 ยท Source

impl Copy for PhantomPinned

1.0.0 ยท Source

impl Copy for RangeFull

1.36.0 ยท Source

impl Copy for RawWakerVTable

1.0.0 ยท Source

impl Copy for RecvError

1.12.0 ยท Source

impl Copy for RecvTimeoutError

Source

impl Copy for SearchStep

1.0.0 ยท Source

impl Copy for SeekFrom

1.0.0 ยท Source

impl Copy for Shutdown

Source

impl Copy for Sign

1.0.0 ยท Source

impl Copy for Sink

1.0.0 ยท Source

impl Copy for SocketAddr

1.0.0 ยท Source

impl Copy for SocketAddrV4

1.0.0 ยท Source

impl Copy for SocketAddrV6

1.28.0 ยท Source

impl Copy for System

Source

impl Copy for SystemRng

1.8.0 ยท Source

impl Copy for SystemTime

1.19.0 ยท Source

impl Copy for ThreadId

1.59.0 ยท Source

impl Copy for TryFromCharError

1.34.0 ยท Source

impl Copy for TryFromIntError

1.34.0 ยท Source

impl Copy for TryFromSliceError

1.0.0 ยท Source

impl Copy for TryRecvError

1.0.0 ยท Source

impl Copy for TypeId

Source

impl Copy for UCred

Available on Unix only.
1.0.0 ยท Source

impl Copy for Utf8Error

1.5.0 ยท Source

impl Copy for WaitTimeoutResult

1.27.0 ยท Source

impl Copy for __m128

1.27.0 ยท Source

impl Copy for __m256

1.72.0 ยท Source

impl Copy for __m512

1.89.0 ยท Source

impl Copy for __m128bh

1.27.0 ยท Source

impl Copy for __m128d

1.94.0 ยท Source

impl Copy for __m128h

1.27.0 ยท Source

impl Copy for __m128i

1.89.0 ยท Source

impl Copy for __m256bh

1.27.0 ยท Source

impl Copy for __m256d

1.94.0 ยท Source

impl Copy for __m256h

1.27.0 ยท Source

impl Copy for __m256i

1.89.0 ยท Source

impl Copy for __m512bh

1.72.0 ยท Source

impl Copy for __m512d

1.94.0 ยท Source

impl Copy for __m512h

1.72.0 ยท Source

impl Copy for __m512i

Source

impl Copy for __tile1024i

Source

impl Copy for bf16

1.0.0 ยท Source

impl Copy for bool

1.0.0 ยท Source

impl Copy for char

1.0.0 ยท Source

impl Copy for f16

1.0.0 ยท Source

impl Copy for f32

1.0.0 ยท Source

impl Copy for f64

1.0.0 ยท Source

impl Copy for f128

1.0.0 ยท Source

impl Copy for i8

1.0.0 ยท Source

impl Copy for i16

1.0.0 ยท Source

impl Copy for i32

1.0.0 ยท Source

impl Copy for i64

1.0.0 ยท Source

impl Copy for i128

1.0.0 ยท Source

impl Copy for isize

1.0.0 ยท Source

impl Copy for u8

1.0.0 ยท Source

impl Copy for u16

1.0.0 ยท Source

impl Copy for u32

1.0.0 ยท Source

impl Copy for u64

1.0.0 ยท Source

impl Copy for u128

1.0.0 ยท Source

impl Copy for usize

1.28.0 ยท Source

impl<'a> Copy for Ancestors<'a>

1.0.0 ยท Source

impl<'a> Copy for Arguments<'a>

1.0.0 ยท Source

impl<'a> Copy for Component<'a>

1.36.0 ยท Source

impl<'a> Copy for IoSlice<'a>

1.10.0 ยท Source

impl<'a> Copy for Location<'a>

Source

impl<'a> Copy for PhantomContravariantLifetime<'a>

Source

impl<'a> Copy for PhantomCovariantLifetime<'a>

Source

impl<'a> Copy for PhantomInvariantLifetime<'a>

1.0.0 ยท Source

impl<'a> Copy for Prefix<'a>

1.0.0 ยท Source

impl<'a> Copy for PrefixComponent<'a>

Source

impl<'a> Copy for Utf8Pattern<'a>

1.63.0 ยท Source

impl<'fd> Copy for BorrowedFd<'fd>

Available on Hermit or Motor OS or Trusty or Unix or WASI only.
1.63.0 ยท Source

impl<'handle> Copy for BorrowedHandle<'handle>

Available on Windows only.
1.63.0 ยท Source

impl<'socket> Copy for BorrowedSocket<'socket>

Available on Windows only.
1.55.0 ยท Source

impl<B, C> Copy for ControlFlow<B, C>
where B: Copy, C: Copy,

Source

impl<Dyn> Copy for DynMetadata<Dyn>
where Dyn: ?Sized,

1.28.0 ยท Source

impl<F> Copy for RepeatWith<F>
where F: Copy,

1.96.0 ยท Source

impl<Idx> Copy for Range<Idx>
where Idx: Copy,

1.96.0 ยท Source

impl<Idx> Copy for RangeFrom<Idx>
where Idx: Copy,

1.95.0 ยท Source

impl<Idx> Copy for RangeInclusive<Idx>
where Idx: Copy,

1.0.0 ยท Source

impl<Idx> Copy for RangeTo<Idx>
where Idx: Copy,

1.26.0 ยท Source

impl<Idx> Copy for std::ops::RangeToInclusive<Idx>
where Idx: Copy,

1.96.0 ยท Source

impl<Idx> Copy for std::range::RangeToInclusive<Idx>
where Idx: Copy,

Source

impl<P> Copy for MaybeDangling<P>
where P: Copy + ?Sized,

1.33.0 ยท Source

impl<Ptr> Copy for Pin<Ptr>
where Ptr: Copy,

1.0.0 ยท Source

impl<T, E> Copy for Result<T, E>
where T: Copy, E: Copy,

Source

impl<T, const N: usize> Copy for Mask<T, N>
where T: MaskElement,

Source

impl<T, const N: usize> Copy for Simd<T, N>
where T: SimdElement,

1.58.0 ยท Source

impl<T, const N: usize> Copy for [T; N]
where T: Copy,

Source

impl<T, const VARIANT: u32, const FIELD: u32> Copy for FieldRepresentingType<T, VARIANT, FIELD>
where T: ?Sized,

1.0.0 ยท Source

impl<T: Copy> Copy for SendError<T>

Source

impl<T: Copy> Copy for SendTimeoutError<T>

1.0.0 ยท Source

impl<T: Copy> Copy for TrySendError<T>

1.0.0 ยท Source

impl<T> Copy for &T
where T: ?Sized,

Shared references can be copied, but mutable references cannot!

1.0.0 ยท Source

impl<T> Copy for *const T
where T: ?Sized,

1.0.0 ยท Source

impl<T> Copy for *mut T
where T: ?Sized,

1.17.0 ยท Source

impl<T> Copy for Bound<T>
where T: Copy,

1.21.0 ยท Source

impl<T> Copy for Discriminant<T>

1.20.0 ยท Source

impl<T> Copy for ManuallyDrop<T>
where T: Copy + ?Sized,

1.36.0 ยท Source

impl<T> Copy for MaybeUninit<T>
where T: Copy,

1.25.0 ยท Source

impl<T> Copy for NonNull<T>
where T: ?Sized,

1.28.0 ยท Source

impl<T> Copy for NonZero<T>

1.0.0 ยท Source

impl<T> Copy for Option<T>
where T: Copy,

Source

impl<T> Copy for PhantomContravariant<T>
where T: ?Sized,

Source

impl<T> Copy for PhantomCovariant<T>
where T: ?Sized,

1.0.0 ยท Source

impl<T> Copy for PhantomData<T>
where T: ?Sized,

Source

impl<T> Copy for PhantomInvariant<T>
where T: ?Sized,

1.36.0 ยท Source

impl<T> Copy for Poll<T>
where T: Copy,

1.19.0 ยท Source

impl<T> Copy for Reverse<T>
where T: Copy,

1.74.0 ยท Source

impl<T> Copy for Saturating<T>
where T: Copy,

Source

impl<T> Copy for SyncView<T>
where T: Sync + Copy,

1.0.0 ยท Source

impl<T> Copy for Wrapping<T>
where T: Copy,

Source

impl<Y, R> Copy for CoroutineState<Y, R>
where Y: Copy, R: Copy,